package com.glxp.udidl.admin.controller.device;

import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udi.UdiCompanyEntity;
import com.glxp.udidl.admin.entity.udid.UdiEntity;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.req.UdiCompanyRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.res.udid.ProductInfoResponse;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.UdiCompanyService;
import com.glxp.udidl.admin.thread.AsyncDownloadTask;
import com.glxp.udidl.admin.util.FilterUdiUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

//udi出入库管理系统API
@Api(tags = "产品信息接口")
@RestController
public class ProductInfoController {

    @Resource
    ProductInfoService productInfoService;
    @Resource
    UdiCompanyService udiCompanyService;
    @Resource
    AsyncDownloadTask asyncDownloadTask;

    //手持枪扫码查询
    @ApiOperation(value = "手持枪扫码查询", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/device/findBydi")
    @GetMapping("udidl/device/findBydi")
    public BaseResponse findProductInfo(ProductInfoFilterRequest productInfoFilterRequest) {
        String key = productInfoFilterRequest.getNameCode();
        if (key != null) {
            key = FilterUdiUtils.getDiStr(key);
            productInfoFilterRequest.setNameCode(key);
        }
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        ProductInfoResponse productInfoResponse = new ProductInfoResponse();
        if (productInfoEntityList != null && productInfoEntityList.size() > 0) {
            ProductInfoEntity productInfoEntity = productInfoEntityList.get(productInfoEntityList.size() - 1);
            productInfoResponse.setProductInfoEntity(productInfoEntity);
            ProductInfoFilterRequest packFilter = new ProductInfoFilterRequest();
            packFilter.setUuid(productInfoEntity.getUuid());
            List<ProductInfoEntity> packInfos = productInfoService.findAll(packFilter);
            List<ProductInfoEntity> returns = new ArrayList<>();
            if (packInfos != null && packInfos.size() > 0) {
                for (ProductInfoEntity temp : packInfos) {
                    if (!temp.getNameCode().equals(productInfoEntity.getNameCode())) {
                        returns.add(temp);
                    }
                }
                productInfoResponse.setPackages(returns);
            }
            return ResultVOUtils.success(productInfoResponse);
        } else {
            if (key != null && key.length() >= 13)
                asyncDownloadTask.downloadByDi(key);    //后台自动下载
            return ResultVOUtils.error(500, "未找到产品信息");
        }
    }

    //管理平台扫码查询
    @ApiOperation(value = "管理平台扫码查询", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/wms/findBydi")
    @GetMapping("udidl/wms/findBydi")
    public BaseResponse findBydiWms(ProductInfoFilterRequest productInfoFilterRequest) {
        String key = productInfoFilterRequest.getNameCode();
        UdiEntity udiEntity = null;
        if (key != null) {
            udiEntity = FilterUdiUtils.getUdi(key);
            key = udiEntity.getUdi();// FilterUdiUtils.getDiStr(key);

            productInfoFilterRequest.setNameCode(key);
        }
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        ProductInfoResponse productInfoResponse = new ProductInfoResponse();
        if (productInfoEntityList != null && productInfoEntityList.size() > 0) {
            ProductInfoEntity productInfoEntity = productInfoEntityList.get(productInfoEntityList.size() - 1);
            if (udiEntity != null) {
                productInfoEntity.setBatchNo(udiEntity.getBatchNo());
                productInfoEntity.setProduceDate(udiEntity.getProduceDate());
                productInfoEntity.setExpireDate(udiEntity.getExpireDate());
            }
            productInfoResponse.setProductInfoEntity(productInfoEntity);
            ProductInfoFilterRequest packFilter = new ProductInfoFilterRequest();
            packFilter.setUuid(productInfoEntity.getUuid());
            List<ProductInfoEntity> packInfos = productInfoService.findAll(packFilter);
            List<ProductInfoEntity> returns = new ArrayList<>();
            if (packInfos != null && packInfos.size() > 0) {
                for (ProductInfoEntity temp : packInfos) {
                    returns.add(temp);
                    if (udiEntity != null) {
                        productInfoEntity.setBatchNo(udiEntity.getBatchNo());
                        productInfoEntity.setProduceDate(udiEntity.getProduceDate());
                        productInfoEntity.setExpireDate(udiEntity.getExpireDate());
                    }
                }
                productInfoResponse.setPackages(returns);
            }
            return ResultVOUtils.success(productInfoResponse);
        } else {
            if (key != null && key.length() >= 13)
                asyncDownloadTask.downloadByDi(key);    //后台自动下载
            return ResultVOUtils.error(500, "未找到产品信息");
        }
    }

    //出入库同步下载
    @ApiOperation(value = "出入库同步下载", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/udiwms/syncUdi")
    @GetMapping("udidl/udiwms/syncUdi")
    public BaseResponse syncUdi(ProductInfoFilterRequest productInfoFilterRequest) {
        List<ProductInfoEntity> productInfoEntityList = productInfoService.syncDlUdi(productInfoFilterRequest);
        return ResultVOUtils.success(productInfoEntityList);
    }

    //出入库同步下载
    @AuthRuleAnnotation("udidl_company_all")
    @GetMapping("udidl/udiwms/syncCompany")
    public BaseResponse syncCompany(UdiCompanyRequest udiCompanyRequest) {
        List<UdiCompanyEntity> udiCompanyEntities = udiCompanyService.syncDlCompany(udiCompanyRequest);
        return ResultVOUtils.success(udiCompanyEntities);
    }

    //开放查询 查询只查询主DI，
    @ApiOperation(value = "根据主DI查询产品信息", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/device/filterUdi")
    @GetMapping("udidl/device/filterUdi")
    public BaseResponse filterUdi(ProductInfoFilterRequest productInfoFilterRequest) {
        //过滤--1.参数不能全未空
        if ((productInfoFilterRequest.getYlqxzcrbarmc() == null || productInfoFilterRequest.getYlqxzcrbarmc().equals("")) &&
                (productInfoFilterRequest.getCpmctymc() == null || productInfoFilterRequest.getCpmctymc().equals("")) &&
                (productInfoFilterRequest.getNameCode() == null || productInfoFilterRequest.getNameCode().equals("")) &&
                (productInfoFilterRequest.getGgxh() == null || productInfoFilterRequest.getGgxh().equals("")) &&
                (productInfoFilterRequest.getUuid() == null || productInfoFilterRequest.getUuid().equals("")) &&
                (productInfoFilterRequest.getZczbhhzbapzbh() == null || productInfoFilterRequest.getZczbhhzbapzbh().equals("")) &&
                (productInfoFilterRequest.getDeviceRecordKey() == null || productInfoFilterRequest.getDeviceRecordKey().equals(""))
        ) {
            return ResultVOUtils.error(500, "参数不能为空！");
        }

        if (productInfoFilterRequest.getNameCode() != null && !productInfoFilterRequest.getNameCode().equals("") && productInfoFilterRequest.getNameCode().length() < 10) {
            return ResultVOUtils.error(500, "请输入完整的最小销售单元标识！");
        }

        if (productInfoFilterRequest.getYlqxzcrbarmc() != null && !productInfoFilterRequest.getYlqxzcrbarmc().equals("") && productInfoFilterRequest.getYlqxzcrbarmc().length() < 4) {
            return ResultVOUtils.error(500, "请输入完整的医疗器械注册备案人名称！");
        }


        if ((productInfoFilterRequest.getYlqxzcrbarmc() == null || productInfoFilterRequest.getYlqxzcrbarmc().equals(""))
                && (productInfoFilterRequest.getNameCode() == null || productInfoFilterRequest.getNameCode().equals(""))
                && (productInfoFilterRequest.getCpmctymc() == null || productInfoFilterRequest.getCpmctymc().equals(""))

        ) {
            if (productInfoFilterRequest.getGgxh() != null && !productInfoFilterRequest.getGgxh().equals("")) {
                return ResultVOUtils.error(500, "请输入医疗器械注册备案人名称");
            }
            if (productInfoFilterRequest.getZczbhhzbapzbh() != null && !productInfoFilterRequest.getZczbhhzbapzbh().equals("")) {
                return ResultVOUtils.error(500, "请输入医疗器械注册备案人名称");
            }
        }

       /* if (productInfoFilterRequest.getCpmctymc() != null && !productInfoFilterRequest.getCpmctymc().equals("") && productInfoFilterRequest.getCpmctymc().length() < 4) {
            return ResultVOUtils.error(500, "请输入完整的产品通用名称！");
        }*/
        productInfoFilterRequest.setIsNewest(1);
        productInfoFilterRequest.setDiType("1");
        List<ProductInfoEntity> productInfoEntityList = productInfoService.filterUdi(productInfoFilterRequest);
        PageInfo<ProductInfoEntity> pageInfo;
        pageInfo = new PageInfo<>(productInfoEntityList);
        PageSimpleResponse<ProductInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(productInfoEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    //开放查询 查询全部，不会过滤只显示主
    @ApiOperation(value = "查询全部产品信息接口", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/device/filterAllUdi")
    @GetMapping("udidl/device/filterAllUdi")
    public BaseResponse filterAllUdi(ProductInfoFilterRequest productInfoFilterRequest) {
        //过滤--1.参数不能全未空
        if ((productInfoFilterRequest.getYlqxzcrbarmc() == null || productInfoFilterRequest.getYlqxzcrbarmc().equals("")) &&
                (productInfoFilterRequest.getCpmctymc() == null || productInfoFilterRequest.getCpmctymc().equals("")) &&
                (productInfoFilterRequest.getNameCode() == null || productInfoFilterRequest.getNameCode().equals("")) &&
                (productInfoFilterRequest.getGgxh() == null || productInfoFilterRequest.getGgxh().equals("")) &&
                (productInfoFilterRequest.getUuid() == null || productInfoFilterRequest.getUuid().equals("")) &&
                (productInfoFilterRequest.getZczbhhzbapzbh() == null || productInfoFilterRequest.getZczbhhzbapzbh().equals("")) &&
                (productInfoFilterRequest.getDeviceRecordKey() == null || productInfoFilterRequest.getDeviceRecordKey().equals(""))
        ) {
            return ResultVOUtils.error(500, "参数不能为空！");
        }

        if (productInfoFilterRequest.getNameCode() != null && !productInfoFilterRequest.getNameCode().equals("") && productInfoFilterRequest.getNameCode().length() < 10) {
            return ResultVOUtils.error(500, "请输入完整的最小销售单元标识！");
        }

        if (productInfoFilterRequest.getYlqxzcrbarmc() != null && !productInfoFilterRequest.getYlqxzcrbarmc().equals("") && productInfoFilterRequest.getYlqxzcrbarmc().length() < 4) {
            return ResultVOUtils.error(500, "请输入完整的医疗器械注册备案人名称！");
        }


        if ((productInfoFilterRequest.getYlqxzcrbarmc() == null || productInfoFilterRequest.getYlqxzcrbarmc().equals(""))
                && (productInfoFilterRequest.getNameCode() == null || productInfoFilterRequest.getNameCode().equals(""))
                && (productInfoFilterRequest.getCpmctymc() == null || productInfoFilterRequest.getCpmctymc().equals(""))

        ) {
            if (productInfoFilterRequest.getGgxh() != null && !productInfoFilterRequest.getGgxh().equals("")) {
                return ResultVOUtils.error(500, "请输入医疗器械注册备案人名称");
            }
            if (productInfoFilterRequest.getZczbhhzbapzbh() != null && !productInfoFilterRequest.getZczbhhzbapzbh().equals("")) {
                return ResultVOUtils.error(500, "请输入医疗器械注册备案人名称");
            }
        }

        if (productInfoFilterRequest.getCpmctymc() != null && !productInfoFilterRequest.getCpmctymc().equals("") && productInfoFilterRequest.getCpmctymc().length() < 4) {
            return ResultVOUtils.error(500, "请输入完整的产品通用名称！");
        }
        productInfoFilterRequest.setIsNewest(1);
        List<ProductInfoEntity> productInfoEntityList = productInfoService.filterUdi(productInfoFilterRequest);
        PageInfo<ProductInfoEntity> pageInfo;
        pageInfo = new PageInfo<>(productInfoEntityList);
        PageSimpleResponse<ProductInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(productInfoEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @ApiOperation(value = "根据UUID查询产品信息", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/device/filterByUuid")
    @GetMapping("udidl/device/filterByUuid")
    public BaseResponse filterByUuid(ProductInfoFilterRequest productInfoFilterRequest) {
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        return ResultVOUtils.success(productInfoEntityList);
    }

    @ApiOperation(value = "根据最小标识查询产品信息", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/device/filterByNameCode")
    @GetMapping("udidl/device/filterByNameCode")
    public BaseResponse filterByNameCode(ProductInfoFilterRequest productInfoFilterRequest) {
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        Map<String, List<ProductInfoEntity>> listMap = productInfoEntityList.stream().collect(Collectors.groupingBy(ProductInfoEntity::getUuid));
        List<ProductInfoEntity> result = null;
        for (List<ProductInfoEntity> value : listMap.values()) {
            if (result == null || value.get(0).getVersionNumber() > result.get(0).getVersionNumber()) {
                result = value;
            }
        }
        productInfoFilterRequest.setNameCode(null);
        productInfoFilterRequest.setUuid(result.get(0).getUuid());
        List<ProductInfoEntity> datas = productInfoService.findAll(productInfoFilterRequest);
        return ResultVOUtils.success(datas);
    }

}
