package com.glxp.udidl.admin.controller.device;

import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udid.DeviceEntity;
import com.glxp.udidl.admin.req.udid.DeviceRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.udid.DataSetSingleHistoryResult;
import com.glxp.udidl.admin.service.dataSync.DeviceSyncService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;

@Api(tags = "下载医疗器械信息接口")
@RestController
public class UdidlDeviceController {

    private static final Logger logger = LoggerFactory.getLogger(UdidlDeviceController.class);

    @Autowired
    DeviceSyncService deviceSyncService;

    @ApiOperation(value = "按日期段下载医疗器械信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "startDate", value = "开始时间", required = true, dataType = "Date"),
            @ApiImplicitParam(name = "endDate", value = "结束时间", required = true, dataType = "Date")})
    @AuthRuleAnnotation("udidl/device/dlByDays")
    @GetMapping("udidl/device/dlByDays")
    public BaseResponse dlByDays(@DateTimeFormat(pattern = "yyyy-MM-dd") Date startDate, @DateTimeFormat(pattern = "yyyy-MM-dd") Date endDate) {
        logger.info("---按日期段下载");
        deviceSyncService.downloadUdi(startDate, endDate);
        return ResultVOUtils.success("后台已开始下载！");
    }

    @ApiOperation(value = "按天下载医疗器械信息")
    @ApiImplicitParam(name = "day", value = "指定日期", required = true, dataType = "String")
    @AuthRuleAnnotation("udidl/device/dlByDay")
    @GetMapping("udidl/device/dlByDay")
    public BaseResponse dlByDay(String day) {
        logger.info(day + "---按天开启下载");
        deviceSyncService.downloadUdi(day, "manual");
        return ResultVOUtils.success("后台已开始下载！");
    }

    //todo 补全返回参数说明
    @ApiOperation(value = "下载指定标识的设备信息", response = DataSetSingleHistoryResult.class)
    @ApiImplicitParam(name = "deviceId", value = "设备ID", required = true, dataType = "String")
    @AuthRuleAnnotation("udidl/device/dlByDi")
    @PostMapping("udidl/device/dlByDi")
    public BaseResponse downloadByDi(String deviceId) {
        return deviceSyncService.downloadByDi(deviceId);
    }


    @AuthRuleAnnotation("udidl_udidlDevice_all")
    @GetMapping("udidl/device/serchDlByDi")
    public BaseResponse serchDlByDi(String deviceId) {
        return deviceSyncService.searchDlByDi(deviceId);
    }

    /**
     * 获取产品标识详情
     *
     * @param deviceId
     * @return
     */
    @ApiOperation(value = "下载单个产品详情", response = DeviceEntity.class)
    @ApiImplicitParam(name = "deviceId", value = "医疗器械设备ID", required = true, dataType = "String")
    @AuthRuleAnnotation("udidl/device/downloadSingle")
    @PostMapping("udidl/device/downloadSingle")
    public BaseResponse downloadSingle(String deviceId) {
        return deviceSyncService.downloadSingle(deviceId);
    }

    @ApiOperation(value = "下载产品历史信息", response = DeviceEntity.class)
    @ApiImplicitParam(name = "key", value = "关键字")
    @AuthRuleAnnotation("udidl/device/downloadHistory")
    @PostMapping("udidl/device/downloadHistory")
    public BaseResponse downloadHistory(String key) {
        return deviceSyncService.downloadHistory(key);
    }

    /**
     * 下载医疗器械信息
     *
     * @return
     */
    @ApiOperation(value = "下载国家库产品数据", response = BaseResponse.class)
    @AuthRuleAnnotation("")
    @PostMapping("udidl/device/downloadUdiData")
    public BaseResponse downloadUdiData(@RequestBody DeviceRequest deviceRequest) {
        if (null == deviceRequest) {
            return ResultVOUtils.error(500, "下载参数不能为空");
        }
        deviceRequest.setDownloadType("manual"); //接口手动下载
        deviceSyncService.downloadUdiData(deviceRequest);
        return ResultVOUtils.success("后台已经开始下载");
    }
}
