package com.glxp.udidl.admin.controller.info;

import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.info.CompanyEntity;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.info.CompanyService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

@Api(tags = "企业信息接口")
@RestController
public class CompanyController {

    @Resource
    private CompanyService companyService;

    @ApiOperation(value = "查询企业信息", response = CompanyEntity.class)
    @AuthRuleAnnotation("warehouse/info/company")
    @GetMapping("/warehouse/info/company")
    public BaseResponse getCompanyInfo() {
        CompanyEntity companyEntity = companyService.findCompany();
        if (companyEntity != null) {
            return ResultVOUtils.success(companyEntity);
        }
        return ResultVOUtils.error(500, "企业信息为空");
    }

    @ApiOperation(value = "修改企业信息")
    @AuthRuleAnnotation("warehouse/info/modifyCompany")
    @PostMapping("/warehouse/info/modifyCompany")
    public BaseResponse modifyCompany(@RequestBody CompanyEntity companyEntity) {
        boolean b = companyService.modifyCompany(companyEntity);
        return ResultVOUtils.success("修改成功");
    }

    @ApiOperation(value = "测试国家UDI库接口连通性")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "appId", value = "appId", required = true),
            @ApiImplicitParam(name = "appSecret", value = "秘钥", required = true),
            @ApiImplicitParam(name = "companyName", value = "公司名称", required = true),
            @ApiImplicitParam(name = "creditNum", value = "统一社会信用代码", required = true),
            @ApiImplicitParam(name = "dataSource", value = "账号数据来源：1.UDIMS，2.UDI自助平台", required = true)
    })
    @AuthRuleAnnotation("warehouse/info/testConnection")
    @GetMapping("/warehouse/info/testConnection")
    public BaseResponse testConnection(String appId, String appSecret, String companyName, String creditNum, String dataSource) {
        return companyService.testConnection(appId, appSecret, companyName, creditNum, dataSource);
    }
}
